@@ -27,6 +27,12 @@
 
 #include "gstaudiopack.h"
 
+#ifdef HAVE_ORC
+#include <orc/orcfunctions.h>
+#else
+#define orc_memset memset
+#endif
+
 #if G_BYTE_ORDER == G_LITTLE_ENDIAN
 # define audio_orc_unpack_s16le audio_orc_unpack_s16
 # define audio_orc_unpack_s16be audio_orc_unpack_s16_swap
@@ -398,6 +404,8 @@ gst_audio_format_from_string (const gchar * format)
 {
   guint i;
 
+  g_return_val_if_fail (format != NULL, GST_AUDIO_FORMAT_UNKNOWN);
+
   for (i = 0; i < G_N_ELEMENTS (formats); i++) {
     if (strcmp (GST_AUDIO_FORMAT_INFO_NAME (&formats[i]), format) == 0)
       return GST_AUDIO_FORMAT_INFO_FORMAT (&formats[i]);
@@ -410,7 +418,7 @@ gst_audio_format_to_string (GstAudioFormat format)
 {
   g_return_val_if_fail (format != GST_AUDIO_FORMAT_UNKNOWN, NULL);
 
-  if (format >= G_N_ELEMENTS (formats))
+  if ((gint) format >= G_N_ELEMENTS (formats))
     return NULL;
 
   return GST_AUDIO_FORMAT_INFO_NAME (&formats[format]);
@@ -427,7 +435,7 @@ gst_audio_format_to_string (GstAudioFormat format)
 const GstAudioFormatInfo *
 gst_audio_format_get_info (GstAudioFormat format)
 {
-  g_return_val_if_fail (format < G_N_ELEMENTS (formats), NULL);
+  g_return_val_if_fail ((gint) format < G_N_ELEMENTS (formats), NULL);
 
   return &formats[format];
 }
@@ -453,14 +461,41 @@ gst_audio_format_fill_silence (const GstAudioFormatInfo * info,
   if (info->flags & GST_AUDIO_FORMAT_FLAG_FLOAT ||
       info->flags & GST_AUDIO_FORMAT_FLAG_SIGNED) {
     /* float or signed always 0 */
-    memset (dest, 0, length);
+    orc_memset (dest, 0, length);
   } else {
     gint i, j, bps = info->width >> 3;
 
     switch (bps) {
       case 1:
-        memset (dest, info->silence[0], length);
+        orc_memset (dest, info->silence[0], length);
         break;
+      case 2:{
+#if G_BYTE_ORDER == G_LITTLE_ENDIAN
+        guint16 silence = GST_READ_UINT16_LE (info->silence);
+#else
+        guint16 silence = GST_READ_UINT16_BE (info->silence);
+#endif
+        audio_orc_splat_u16 (dest, silence, length / bps);
+        break;
+      }
+      case 4:{
+#if G_BYTE_ORDER == G_LITTLE_ENDIAN
+        guint32 silence = GST_READ_UINT32_LE (info->silence);
+#else
+        guint32 silence = GST_READ_UINT32_BE (info->silence);
+#endif
+        audio_orc_splat_u32 (dest, silence, length / bps);
+        break;
+      }
+      case 8:{
+#if G_BYTE_ORDER == G_LITTLE_ENDIAN
+        guint64 silence = GST_READ_UINT64_LE (info->silence);
+#else
+        guint64 silence = GST_READ_UINT64_BE (info->silence);
+#endif
+        audio_orc_splat_u64 (dest, silence, length / bps);
+        break;
+      }
       default:
         for (i = 0; i < length; i += bps) {
           for (j = 0; j < bps; j++)
